@@ -1,6 +1,5 @@
-from .package_info.version import VERSION
-
 # Classes and functions
+from .package_info import VERSION, GITHUB, PYPI, github
 from .base import Lattice, Poset, Relation
 from .base_generation.exhaustive import iter_all_lattices
 from .base_generation.random import (
@@ -10,9 +9,8 @@ from .base_generation.random import (
 )
 from .functional import f_lub, f_glb, f_iter
 
-from .package_info.inspect import github, GITHUB, PYPI
-
 # Modules
+from . import package_info
 from . import _enum as enum
 from . import functional
 from .base_methods import validation, graphviz
